"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.map_tool = void 0;
const options_1 = require("../entities/options");
const regex_1 = require("../utils/regex");
const debug_1 = __importDefault(require("../entities/commands/debug"));
const help_1 = __importDefault(require("../entities/commands/help"));
const validation_1 = __importDefault(require("./validation"));
const CommandResolver_1 = require("../entities/CommandResolver");
/**
 * Maps the specified tool to the Robot.
 *
 * @export
 * @param {NodeModule} caller The caller.
 * @param {NodeModule} packageModule The package module.
 * @param {IRobot} robot The robot.
 * @param {ITool} tool The tool that will be mapped.
 * @param {IOptions} [options] The options for this specific mapping.
 */
function map_tool(robot, tool, options = options_1.defaultOptions) {
    if (!robot)
        throw "Argument 'robot' is empty.";
    if (!tool)
        throw "Argument 'tool' is empty.";
    if (!tool.commands)
        tool.commands = [];
    (0, validation_1.default)(tool);
    // add a debug command
    tool.__registrations = [];
    if (options.addDebugCommand) {
        tool.commands.push((0, debug_1.default)());
    }
    //add help
    const helpCommand = (0, help_1.default)();
    if (options.addHelpCommand) {
        tool.commands.push(helpCommand);
    }
    //init every command
    tool.commands.forEach(cmd => {
        //use a second validation regex to confirm the message we
        //are responding to, is as we expected. This will prevent command
        //match edge cases in which certain phrases end with a command name
        const strValidationRegex = (0, regex_1.convertCommandIntoRegexString)(robot.name, robot.alias, tool, cmd);
        cmd.validationRegex = new RegExp(strValidationRegex, "si");
        if (robot.logger) {
            robot.logger.info(`Mapping '${tool.name}.${cmd.name}' as '${strValidationRegex}'.`);
        }
        //needed for the debug command
        tool.__registrations.push({
            commandName: cmd.name,
            messageRegex: strValidationRegex
        });
    });
    //listen for invocation of tool
    const toolRegexString = (0, regex_1.convertToolIntoRegexString)(robot.name, robot.alias, tool);
    const toolRegex = new RegExp(toolRegexString, "i");
    tool.__robotRegex = toolRegex;
    const handler = tool;
    handler.canHandle = (msg) => toolRegex.test(msg);
    // add tool to robot - helps with middleware
    const bot = robot;
    bot.__tools = bot.__tools || [];
    bot.__tools.push(handler);
    const resolver = new CommandResolver_1.CommandResolver(robot);
    robot.respond(toolRegex, res => {
        var action = resolver.resolveFromTool(tool, res);
        if (!action || !action.tool) {
            return;
        }
        //if no commands matched, show help command
        if (action.command == null) {
            if (options.showHelpOnInvalidSyntax) {
                helpCommand.execute({
                    robot,
                    match: null,
                    res,
                    tool,
                    values: {}
                }, options.invalidSyntaxHelpPrefix, options.invalidSyntaxMessage);
            }
            else if (options.showInvalidSyntax) {
                res.reply(options.invalidSyntaxMessage);
            }
            return;
        }
        action.log(bot.logger);
        if (!action.authorized) {
            res.reply(options.notAuthorizedMessage);
            return;
        }
        if (options.replacedByBot) {
            const botRegexStr = (0, regex_1.convertBotNameIntoRegexString)(robot.name, robot.alias);
            const botRegex = new RegExp(botRegexStr);
            const command = `@${options.replacedByBot} ` + action.text.replace(botRegex, "").trim();
            res.reply(`Sorry, this feature has been replaced by <@${options.replacedByBot}>. Please use:\n` +
                "```\n" +
                command +
                "\n```\n");
        }
        else {
            action.command.execute({
                tool: tool,
                robot: robot,
                res: res,
                match: action.match,
                values: action.values
            });
        }
    });
}
exports.map_tool = map_tool;
